use strict;
use warnings;

package RTx::Authen::Launchpad;

use RTx::Authen::Launchpad::Utils qw(trim uniquify);

use RT::Group;
use RT::User;
use RT::CustomField;
use RT::Attribute;
use RT::Interface::Web;
use URI qw();

use constant SREG => 'http://openid.net/extensions/sreg/1.1';
use constant TEAM => 'http://ns.launchpad.net/2007/openid-teams';
use constant LP_TEAMS_CF => 'LaunchpadTeams';

our $VERSION = "0.3.2";

sub new {
    my $class = shift;
    my $self = {};

    bless $self, $class;

    $self->{allowed_teams} = [map { RTx::Authen::Launchpad::Utils::trim($_) } 
                              split /,/, RT->Config->Get('LaunchpadAllowedTeams')];

    $self->{lp_teams_cf} = RT::CustomField->new(RT->SystemUser);
    $self->{lp_teams_cf}->Load(LP_TEAMS_CF);

    $self->{teams_mapping} = $self->_LaunchpadTeamsMapping();

    $self;
}

sub TeamsMapping { $_[0]->{teams_mapping}; }

sub ConfigureClaimedIdentity {
    my $self = shift;
    my $claimed_identity = shift;

    $claimed_identity->set_extension_args(
        SREG, 
        {
            required => 'nickname,email', 
            optional => 'fullname,postcode,country,timezone'
        });

    $claimed_identity->set_extension_args(
        TEAM,
        {
            query_membership => join(",", $self->_QueryMembershipList()),
        });

    $claimed_identity;
}

sub GetCheckUrl {
    my $self = shift;
    my $claimed_identity = shift;
    my $next = URI->new_abs(RT::Interface::Web::IntuitNextPage(), RT->Config->Get('WebBaseURL'));

    $claimed_identity->check_url(
        # Skip check_immediate, as LP doesn't support that
        delayed_return => 1,
        return_to      => $next,
        trust_root     => RT->Config->Get('WebURL'),
        );
}

sub UserIsMemberOfAllowedTeams {
    my $self = shift;
    my $member_of = shift;

    # no team restrictions
    if (not @{$self->{allowed_teams}}) {
        return 1;
    }

    # check restricted teams membership
    foreach my $team (@{$self->{allowed_teams}}) {
        return 1 if grep /^$team$/, @$member_of;
    }
    return 0;
}

sub _QueryMembershipList {
    my $self = shift;
    my @teams = ();
    push @teams, @{$self->{allowed_teams}};

    foreach my $teams_list (values %{$self->{teams_mapping}}) {
        push @teams, @$teams_list;
    }
    
    RTx::Authen::Launchpad::Utils::uniquify(@teams);
}

sub _LaunchpadTeamsMapping {
    my $self = shift;
    my %teams = ();
    
    my $groups = RT::Groups->new(RT->SystemUser);
    $groups->LimitToUserDefinedGroups();

    while (my $group = $groups->Next) {
        my $values = $group->CustomFieldValues($self->{lp_teams_cf});
        while (my $value = $values->Next) {
            push @{$teams{$group->Name}}, $value->Content; 
        }
    }
    \%teams;
}

sub _GetGroup {
    my $self = shift;
    my $group_name = shift;

    my $group = RT::Group->new(RT->SystemUser);
    $group->LoadUserDefinedGroup($group_name);
    
    $group;
}

sub _RemoveUserFromGroupsAssignedOnLastLogin {
    my $self = shift;
    my $user = $self->_GetUser(shift);
    my @groups_previously_assigned = $self->GroupNamesAssignedToUser($user);
    
    foreach my $group_name (@groups_previously_assigned) {
        my $group = $self->_GetGroup($group_name);
        $group->DeleteMember($user) if $group->HasMember($user);
    }
}

sub AssignUserToGroups {
    my $self = shift;
    my $user_id = shift;
    my $member_of = shift;
    my @groups_assigned_to = ();

    $self->_RemoveUserFromGroupsAssignedOnLastLogin($user_id);

    foreach my $group_name (keys %{$self->{teams_mapping}}) {
        foreach my $team (@$member_of) {
            if (grep /^$team$/, @{$self->{teams_mapping}->{$group_name}}) {
                my $group = $self->_GetGroup($group_name);
                if ($group and not $group->HasMember($user_id)) {
                    my ($id, $msg) = $group->AddMember($user_id);
                    push @groups_assigned_to, $group_name;
                    $RT::Logger->info($msg) if $msg;
                }
            }
        }
    }
    $self->SaveAssignedGroups($user_id, @groups_assigned_to);
}

sub _ClearAssignedGroups {
    my $self = shift;
    my $user = shift;

    my $attr = RT::Atribute->new(RT->SystemUser);
    my ($rv, $msg) = $attr->LoadByNameAndObject(Object => $user, Name => 'assigned_groups');

    $attr->Delete if $rv;
}

sub GroupNamesAssignedToUser {
    my $self = shift;
    my $user = shift;

    my $attr = $self->_GetOrCreateAssignedGroups($user);
    return map { RTx::Authen::Launchpad::Utils::trim($_) } split /,/, $attr->Content;
}

sub _GetOrCreateAssignedGroups {
    my $self = shift;
    my $user = shift;
    
    my $attr = RT::Attribute->new(RT->SystemUser);

    my ($rv, $msg) = $attr->LoadByNameAndObject(
        Object => $user, 
        Name => 'assigned_groups',
        );    

    if ($rv == 0) {
        $attr->Create(
            Object => $user,
            Name   => 'assigned_groups',
            );
    }

    return $attr;
}

sub _GetUser {
    my $self = shift;
    my $user_id = shift;

    my $user = RT::User->new(RT->SystemUser);
    $user->Load($user_id);
    
    return $user;
}

sub SaveAssignedGroups {
    my $self = shift;
    my $user_id = shift;
    my @groups_assigned_to = @_;

    my $user = $self->_GetUser($user_id);

    my $attr = $self->_GetOrCreateAssignedGroups($user);
    $attr->SetContent(join(',', @groups_assigned_to));
}

1;
